clear;
% This code computes the driver dominant scores of each gene in the set of
% all genes being analysed
day=clock;
disp(datestr(datenum(day(1),day(2),day(3),day(4),day(5),day(6)),0))
CNV=abs(dlmread('PartialCovMatrixCN.txt'));  %Call the partial covariance matrix estimated from Copy Number(CN) data.
EXP=abs(dlmread('PartialCovMatrixEXP.txt'));  %Call the partial covariance matrix estimated from Gene Expression (EXP) data.
CNVandEXP=abs(dlmread('PartialCovMatrixCNandEXP.txt')); %Call the partial covariance matrix estimated across Copy Number and Gene Expression data.
MDImatrix=MDI_Matrix(CNV,EXP,CNVandEXP); % Construct a Multi-Type Interaction(MTI) network by obtaining an optimal linear combination of the three matrices. 
CNV=[];EXP=[];CNVandEXP=[];
beta=0.5;
n=length(MDImatrix(1,:));
Corr=[];
InWeight=sum(abs(MDImatrix)); %Compute the total incoming influence weight of each gene on MTI network.
OutWeight=sum(abs(MDImatrix')); %Compute the total outgoing influence weight of each gene on MTI network.
%Compute the direct incoming and outgoing effect of each gene
%_____________________________________________________
%
for k=1:n
    Effect(k)=OutWeight(k)/(1+InWeight(k));
    InDeg(k)=length(find(MDImatrix(:,k)));
    InEff(k)=InDeg(k)*(InWeight(k)/InDeg(k))^beta; %Compute the total incoming effect of each gene on MTI network based on incoming degree and weight 
    OutDeg(k)=length(find(MDImatrix(k,:)));
    if OutDeg(k)==0
       OutDeg(k)=1;
    end
    OutEff(k)=OutDeg(k)*(OutWeight(k)/OutDeg(k))^beta; %Compute the total outgoing effect of each gene on MTI network based on outgoing degree and weight 
end
%Compute the total incoming and outgoing effect of each gene
%_____________________________________________________
%
MeanIncomingW=mean(InWeight);
GeneScore=zeros(n,1);
for i=1:n
    UpS=InWeight(i);
    DownS=OutWeight(i);
    if DownS>MeanIncomingW+UpS
       distance=dfs(MDImatrix,i);
        for j=1:n
           if distance(j)>100
              distance(j)=100; 
           end
           if distance(j)==-1 
              distance(j)=1000000;
           end
        end
    Cov=MDImatrix;  
    Cov(i,i)=Cov(i,i)+1;
    DownS=TreeScore(i,Cov,distance,OutEff,beta);
    GeneScore(i)=DownS/(1+UpS);
    end
    disp(['GENE' num2str(i) ' gene score= ' num2str(GeneScore(i))])
end
[Scores,indexs]=sort(GeneScore,'descend');
GeneScores=num2cell(Scores');
[~,GeneSymbols]=xlsread('GeneOnPPI.xlsx');
GeneScore=[GeneSymbols(indexs),GeneScores'];
DrIndex=find(Scores); 
xlswrite('KeyPlayers.xlsx',GeneScore(DrIndex,:));
day=clock;
disp(datestr(datenum(day(1),day(2),day(3),day(4),day(5),day(6)),0))